package idi.gate.associationrules;


import idi.gate.associationrules.generator.AssociationRulesGenerator;
import idi.gate.associationrules.generator.FrequentItemSetsGenerator;
import idi.gate.associationrules.rule.Rule;
import idi.gate.index.Document;
import idi.gate.index.Index;
import idi.gate.index.StopWords;

import java.io.IOException;
import java.net.URL;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeMap;

import gate.Gate;
import gate.LanguageResource;
import gate.ProcessingResource;
import gate.Resource;
import gate.creole.AbstractLanguageAnalyser;
import gate.creole.ExecutionException;
import gate.creole.ResourceInstantiationException;

@SuppressWarnings("serial")
public class AssociationRulesWrapper extends AbstractLanguageAnalyser 
	implements ProcessingResource {
	
	private String inputAnnotationSetName;
	private String annotationName;
	private Integer nofFrequentTerms;
	private Double minimumSupport;
	private Double miniumConfidence;
	private URL stopWordListFile;
	private Index index;
	private long startTime;
	private Configuration config;
	private FrequentItemSetsGenerator frequentItemSetsGenerator;
	private AssociationRulesGenerator associationRulesGenerator;
	private ArrayList<Rule> associationRules;
	private StopWords stopWords;
	private ArrayList<String> stopWordList;
	
	public void execute() throws ExecutionException {
		if ( corpus == null){
			throw new ExecutionException("Corpus is not initialized");
		}
		index = Index.getInstance();
		index.clearIndex();
		config = Configuration.getInstance();
		config.setAnnotationName(annotationName);
		config.setInputAnnotationSetName(inputAnnotationSetName);
		config.setNofFrequentTerms(nofFrequentTerms);
		config.setMinimumSupport(minimumSupport.floatValue());
		config.setMiniumConfidence(miniumConfidence.floatValue());
		
		if(!stopWordListFile.equals("") || !stopWordList.equals("file:/")){
			try {
				stopWords = new StopWords(stopWordListFile);
				stopWordList = stopWords.getStopWordList();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		else stopWordList = new ArrayList<String>();
		
		config.setStopWordList(stopWordList);
		
		
		getDocuments();
		buildIndex();
		startTime = System.currentTimeMillis();
		buildAssociations();
		System.out.println("Generated itemsets and rules in " 
				+ NumberFormat.getInstance().format((double)(System.currentTimeMillis() - startTime) / 1000)
				+ " seconds.");
		fireProcessFinished();
	}

	public Resource init() throws ResourceInstantiationException {
		return this;
	}

	public void reInit() throws ResourceInstantiationException {
		init();
	}
	
	private void getDocuments() {
		Iterator iter = corpus.iterator();
		int counter = 0;
		while(iter.hasNext()){
			fireProgressChanged(10);
			startTime = System.currentTimeMillis();
			fireStatusChanged("Processing document...");
			gate.Document gateDoc = (gate.Document) iter.next();
			Document doc = new Document(gateDoc, counter);
			fireProgressChanged(100);
			fireStatusChanged(gateDoc.getName() + " processed in " + 
					NumberFormat.getInstance().format((double)(System.currentTimeMillis() - startTime) / 1000) + 
					" seconds.");
			counter++;
		}
		fireProgressChanged(0);
	}
	
	private void buildIndex() {
		startTime = System.currentTimeMillis();
		fireProgressChanged(30);
		index.buildIndex();
		fireProgressChanged(100);
		fireStatusChanged("Index build in " + NumberFormat.getInstance().format((double)(System.currentTimeMillis() - startTime) / 1000) + " seconds.");		
	}
	
	private void buildAssociations() {
		frequentItemSetsGenerator = new FrequentItemSetsGenerator();
		fireStatusChanged("Building frequent itemsets...");
		fireProgressChanged(30);
		ArrayList<TreeMap<String, ArrayList<Integer>>> itemSets = 
			frequentItemSetsGenerator.getListOfFrequentItemSets();
		fireProgressChanged(100);
		associationRulesGenerator = new AssociationRulesGenerator(itemSets);
		fireStatusChanged("Building association rules...");
		fireProgressChanged(30);
		associationRulesGenerator.buildAssociations();
		associationRules = associationRulesGenerator.getAssociationRules();
		fireProgressChanged(100);
	}
	

	// Getter and setter methods

	public String getInputAnnotationSetName() {
		return inputAnnotationSetName;
	}

	public void setInputAnnotationSetName(String inputAnnotationSetName) {
		this.inputAnnotationSetName = inputAnnotationSetName;
	}

	public String getAnnotationName() {
		return annotationName;
	}

	public void setAnnotationName(String annotationName) {
		this.annotationName = annotationName;
	}

	public Integer getNofFrequentTerms() {
		return nofFrequentTerms;
	}

	public void setNofFrequentTerms(Integer nofFrequentTerms) {
		this.nofFrequentTerms = nofFrequentTerms;
	}

	public Double getMinimumSupport() {
		return minimumSupport;
	}

	public void setMinimumSupport(Double minimumSupport) {
		this.minimumSupport = minimumSupport;
	}

	public Double getMiniumConfidence() {
		return miniumConfidence;
	}

	public void setMiniumConfidence(Double miniumConfidence) {
		this.miniumConfidence = miniumConfidence;
	}

	public ArrayList<Rule> getAssociationRules() {
		return associationRules;
	}

	public URL getStopWordListFile() {
		return stopWordListFile;
	}

	public void setStopWordListFile(URL stopWordListFile) {
		this.stopWordListFile = stopWordListFile;
	}

	
}
