package idi.gate.nlp.nounphrase;


import gate.Annotation;
import gate.AnnotationSet;
import gate.Corpus;
import gate.Document;
import gate.Factory;
import gate.FeatureMap;
import gate.Gate;
import gate.ProcessingResource;
import gate.creole.ANNIEConstants;
import gate.creole.ResourceInstantiationException;
import gate.creole.SerialAnalyserController;
import gate.util.GateException;
import gate.util.InvalidOffsetException;
import gate.util.OffsetComparator;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;


@SuppressWarnings("unchecked")
public class StandAloneNounPhraseExtractor {

	/** The Corpus Pipeline application to contain ANNIE */
	private SerialAnalyserController annieController;

	/**
	 * Initialise the ANNIE system. This creates a "corpus pipeline" application that can be
	 * used to run sets of documents through the extraction system.
	 */
	public void initAnnie() throws GateException {

		// create a serial analyser controller to run ANNIE with
		annieController = (SerialAnalyserController) Factory.createResource(
				"gate.creole.SerialAnalyserController", Factory.newFeatureMap(),
				Factory.newFeatureMap(), "ANNIE_" + Gate.genSym());

		// load each PR as defined in ANNIEConstants
		for(int i = 0; i < ANNIEConstants.PR_NAMES.length; i++) {
			FeatureMap params = Factory.newFeatureMap(); // use default parameters
			ProcessingResource pr = (ProcessingResource) Factory.createResource(
					ANNIEConstants.PR_NAMES[i], params);

			// add the PR to the pipeline controller
			annieController.add(pr);
		} // for each ANNIE PR

	} // initAnnie()

	/** Tell ANNIE's controller about the corpus you want to run on */
	public void setCorpus(Corpus corpus) {
		annieController.setCorpus(corpus);
	} // setCorpus

	/** Run ANNIE */
	public void execute() throws GateException {
		annieController.execute();
	} // execute()


	public void extractNounPhrase(Corpus corpus) 
		throws GateException, InvalidOffsetException {
		
		Iterator iter = corpus.iterator();
		while(iter.hasNext()){
			Document doc = (Document) iter.next();
			
			System.out.println("\n<EVENT docName='" + doc.getName() + "'>");
			AnnotationSet defaultAnnotSet = doc.getAnnotations();
			Set annotTypesRequired = new HashSet();
			annotTypesRequired.add("Token");
			annotTypesRequired.add("SpaceToken");
			annotTypesRequired.add("Split");
			
			AnnotationSet requiredSet = defaultAnnotSet.get(annotTypesRequired);
			ArrayList<Annotation> annotationList = new ArrayList<Annotation>(requiredSet);
			Collections.sort(annotationList, new OffsetComparator());

			if (requiredSet != null){
				Iterator it = annotationList.iterator();
				Annotation currentAnnotation;
				
				ArrayList<Annotation> outputAnnotationList = new ArrayList<Annotation>();
				boolean insideNounPhrase = false;
				String lastTypeAdded = "";
				String lastWordAdded = "";
				
				while(it.hasNext()){
					currentAnnotation = (Annotation) it.next();
					String feature = null;
					String type = currentAnnotation.getType();
					String word = ""+doc.getContent().getContent(currentAnnotation.getStartNode().getOffset(),
							currentAnnotation.getEndNode().getOffset());

					if(type.equals("Token")) {
						feature = (String) currentAnnotation.getFeatures().get("category");
						if ((feature.equals("NN") || feature.equals("NNP") || feature.equals("NNPS") || 
								feature.equals("NNS") || feature.equals("NP") || feature.equals("NPS")) &&
								!(word.equals("[") || word.equals("]"))){
							insideNounPhrase = true;
							lastTypeAdded = type;
							lastWordAdded = word;
							outputAnnotationList.add(currentAnnotation);
						}
						else if (insideNounPhrase == true){
							/*
							System.out.println("Size: "+outputAnnotationList.size());//1
							System.out.println("Gjeldende ord: "+word); //munciating
							System.out.println("Siste ord: "+lastWordAdded);//com-
							System.out.println("Siste type: "+lastTypeAdded);//SpaceToken
							*/
							if (lastTypeAdded.equals("SpaceToken")) 
								outputAnnotationList.remove(outputAnnotationList.size()-1);
							if (!isPartOfWord(lastWordAdded))
								printAnnotations(outputAnnotationList, doc);
							
							outputAnnotationList.clear();
							lastTypeAdded = "";
							insideNounPhrase = false;	
						}
					}
					else if(type.equals("SpaceToken") && insideNounPhrase == true) {
						if (lastTypeAdded.equals("SpaceToken")) {
							if (!isPartOfWord(lastWordAdded)) {
								insideNounPhrase = false;
								printAnnotations(outputAnnotationList, doc);
								outputAnnotationList.clear();
								lastTypeAdded = "";
							}
							else {
								outputAnnotationList.remove(outputAnnotationList.size()-1);
								lastTypeAdded = "";
							}
						}
						else { 
							outputAnnotationList.add(currentAnnotation);
							lastTypeAdded = type;
						}
					}
					else if(type.equals("Split") && insideNounPhrase == true) {
						if (lastTypeAdded.equals("SpaceToken")) outputAnnotationList.remove(outputAnnotationList.size()-1);
						printAnnotations(outputAnnotationList, doc);
						outputAnnotationList.clear();
						insideNounPhrase = false;
						lastTypeAdded = "";
					}

				}//while
				System.out.println("</EVENT>");
			}
		} //for each doc
	}
	
	private boolean isPartOfWord(String str) {
			char ch = str.charAt(str.length()-1);
			char [] chars = new char[1]; 
			chars[0] = ch;
			String a = new String(chars);
			if (a.equals("")) return true;
			else return false;
	}
	
	
	private void printAnnotations(ArrayList<Annotation> outputAnnotationList, Document doc) throws InvalidOffsetException{
		String ut = "- ";
		for (int i = 0; i < outputAnnotationList.size(); i++) {
			Annotation currentAnnotation = (Annotation) outputAnnotationList.get(i);
			String content = ""+ doc.getContent().getContent(currentAnnotation.getStartNode().getOffset(),
							currentAnnotation.getEndNode().getOffset());
			ut += content;
		}
		System.out.println(ut +"\n");

	}

	public void addDocument(Corpus corpus, String sourceUrl, boolean preserveOriginalContent, boolean collectRepositioningInfo) {
		FeatureMap params = Factory.newFeatureMap();
		params.put("sourceUrl", sourceUrl);
		params.put("preserveOriginalContent", new Boolean(preserveOriginalContent));
		params.put("collectRepositioningInfo", new Boolean(collectRepositioningInfo));

		System.out.println("Creating document: " + sourceUrl);
		Document doc = null;

		try {
			doc = (Document) Factory.createResource("gate.corpora.DocumentImpl", params);
		} catch (ResourceInstantiationException e) {
			e.printStackTrace();
		}
		corpus.add(doc);
	}

	public static void main(String args[]) throws GateException, IOException {
		
		// initialise the GATE library
		Gate.setGateHome(new File("C:\\Program Files\\GATE-3.1\\"));
		Gate.setPluginsHome(new File("C:\\Program Files\\GATE-3.1\\plugins"));
		Gate.init();
		// initialise ANNIE (this may take several minutes)
		StandAloneNounPhraseExtractor annie = new StandAloneNounPhraseExtractor();
		annie.initAnnie();

		// create a GATE corpus and add a document for each command-line
		// argument
		final Corpus corpus = (Corpus) Factory.createResource("gate.corpora.CorpusImpl");
		//C:\\kvarv\\eclipse workspace\\GateTest\\documents\\test.txt
		annie.addDocument(corpus, "file:/C:/Documents and Settings/kvarv.WIN-NTNU-NO/Desktop/dok16.txt", true, true);
		annie.setCorpus(corpus);
		annie.execute();
		annie.extractNounPhrase(corpus);
		

		
	} // main
}
